/**
 * File:
 *  RaceLights.c
 *
 * Notes:
 *  For use with slot racing sets.
 *
 * Ports A:
 *      SW3: 1 OFF
 * Ports B:
 *      SW3: 2 ON
 *
 * History:
 *  09/06/2012 Written by Simon Platten
 */
#include <built_in.h>
#include "types.h"
// Constants
#define BUTTON_DEBOUNCE   20    // time in milliseconds
#define START_BUTTON      0     // Pin on port A
#define STOP_BUTTON       1     // Pin on Port A
#define RESET_BUTTON      2     // Pin on Port A
#define LIGHT_1           0x01  // 1st light
#define LIGHT_2           0x02  // 2nd light
#define LIGHT_3           0x04  // 3rd light
#define LIGHT_4           0x08  // 4th light
#define LIGHT_5           0x10  // 5th light
#define GO_GO_GO          0x20  // Race start signal
#define PAUSED            0x40  // Light sequence paused
// Flag to indicate if race lights running
static volatile boolean blnTimerExpired;
static volatile boolean blnRunning;
static int intStep, intOutCount;
/**
 * Interrupt handler
 */
void interrupt(void) {
  TMR0H = 0x48;    // preset for Timer0 MSB register
  TMR0L = 0xE5;    // preset for Timer0 LSB register
  TMR0IF_bit = 0;

  if ( blnRunning == TRUE && blnTimerExpired == FALSE ) {
    blnTimerExpired = TRUE;
  }
}
/**
 * Resets the lights ready for starting
 */
void resetLights(void) {
// Set the initial state to NOT running
  blnRunning = blnTimerExpired = FALSE;
// Set the light sequence step to the beginning
  intStep = 0;
// All out!
   LATB = 0;
}
/**
 * Entry point
 */
void main(void) {
// Set-up hardware
#ifdef P18F45K22
  ANSELA = 0;                    // Configure PORTA pins as digital
  ANSELB = 0;                    // Configure PORTB pins as digital
#endif
#ifdef P18F4550
  ADCON1 = 0x0f;                 // Configure all ports as digital
#endif
  TRISA = 0xff;                  // Configure PORTA as input
  TRISB = 0;                     // Configure PORTB as output
  LATB  = 0;                     // Clear PORTB
// Reset everything to the initial settings
  resetLights();
// Timer0 Registers:
// 16-Bit Mode; Prescaler=1:128; 
// TMRH Preset=48; TMRL Preset=E5; Freq=1.33333Hz; Period=750.00 ms
  T0CON.TMR0ON = 1;// On/Off Control bit, 1=Enable, 0=Disable
  T0CON.T08BIT = 0;// 8-bit/16-bit Control bi: 0=16-bit timer/counter
  T0CON.T0CS   = 0;// Clock Source Select bit: 0=Internal Clock (CLKO)
  T0CON.T0SE   = 0;// Source Edge Select bit: 0=low/high
  T0CON.PSA    = 0;// Prescaler Assignment bit: 0=Prescaler is assigned; 1=NOT assigned/bypassed
  T0CON.T0PS2  = 1;// bits 2-0  PS2:PS0: Prescaler Select bits
  T0CON.T0PS1  = 0;
  T0CON.T0PS0  = 0;
  TMR0H = 0x48;    // preset for Timer0 MSB register
  TMR0L = 0xE5;    // preset for Timer0 LSB register  
  TMR0IE_bit = 1;
// Global interrupt enable
  GIE_bit = 1;
// Loop forever
  while( 1 ) {
    if ( blnRunning == TRUE ) {
      if ( Button(&PORTA, STOP_BUTTON, BUTTON_DEBOUNCE, 1) ) {
        blnRunning = FALSE;
        LATB |= PAUSED;
      } else if ( blnTimerExpired == TRUE ) {
        switch( intStep ) {
        case 0:         // Light 1
          LATB |= LIGHT_1;
          break;
        case 1:         // Light 2
          LATB |= LIGHT_2;
          break;
        case 2:         // Light 3
          LATB |= LIGHT_3;
          break;
        case 3:         // Light 4
          LATB |= LIGHT_4;
          break;
        case 4:         // Light 5
          LATB |= LIGHT_5;
// Caculate lights out interval
          intOutCount = rand() % (7 - 3) + 3;
          break;
        case 5:         // All out race starts
          if ( --intOutCount == 0 ) {
            resetLights();
            LATB |= GO_GO_GO;
          }
          break;
        }
        if ( (LATB & GO_GO_GO) != GO_GO_GO && intStep < 5 ) {
          intStep++;
        }
// Reset timer flag
        blnTimerExpired = FALSE;
      }
    } else if ( (LATB & GO_GO_GO) != GO_GO_GO && 
                Button(&PORTA, START_BUTTON, BUTTON_DEBOUNCE, 1) ) {
      blnRunning = TRUE;
    } else if ( Button(&PORTA, RESET_BUTTON, BUTTON_DEBOUNCE, 1) ) {
      resetLights();
    }
  }
}